@@ -1,24 +1,28 @@
-{-# LANGUAGE OverlappingInstances
+{-# LANGUAGE CPP
            , FlexibleInstances
            , UndecidableInstances
            #-}
 
+#if __GLASGOW_HASKELL__ < 710
+{-# LANGUAGE OverlappingInstances #-}
+#endif
+
 {-# OPTIONS_GHC -Wall -fwarn-tabs #-}
 
 ----------------------------------------------------------------
---                                                  ~ 2009.01.29
+--                                                  ~ 2021.10.17
 -- |
 -- Module      :  Data.Number.PartialOrd
--- Copyright   :  Copyright (c) 2007--2010 wren ng thornton
+-- Copyright   :  Copyright (c) 2007--2021 wren gayle romano
 -- License     :  BSD3
--- Maintainer  :  wren@community.haskell.org
+-- Maintainer  :  wren@cpan.org
 -- Stability   :  stable
 -- Portability :  semi-portable (OverlappingInstances,...)
--- 
+--
 -- The Prelude's 'Ord' class for dealing with ordered types is often
 -- onerous to use because it requires 'Eq' as well as a total
 -- ordering. While such total orderings are common, partial orderings
--- are moreso. This module presents a class for partially ordered
+-- are more so. This module presents a class for partially ordered
 -- types.
 ----------------------------------------------------------------
 module Data.Number.PartialOrd
@@ -45,49 +49,49 @@ import Hugs.RealFloat (isNaN)
 class PartialOrd a where
     -- | like 'compare'
     cmp   :: a -> a -> Maybe Ordering
-    
+
     -- | like ('>')
     gt    :: a -> a -> Maybe Bool
     gt x y = case x `cmp` y of
              Just GT -> Just True
              Just _  -> Just False
              Nothing -> Nothing
-    
+
     -- | like ('>=')
     ge    :: a -> a -> Maybe Bool
     ge x y = case x `cmp` y of
              Just LT -> Just False
              Just _  -> Just True
              Nothing -> Nothing
-    
+
     -- | like ('==')
     eq    :: a -> a -> Maybe Bool
     eq x y = case x `cmp` y of
              Just EQ -> Just True
              Just _  -> Just False
              Nothing -> Nothing
-    
+
     -- | like ('/=')
     ne    :: a -> a -> Maybe Bool
     ne x y = case x `cmp` y of
              Just EQ -> Just False
              Just _  -> Just True
              Nothing -> Nothing
-    
+
     -- | like ('<=')
     le    :: a -> a -> Maybe Bool
     le x y = case x `cmp` y of
              Just GT -> Just False
              Just _  -> Just True
              Nothing -> Nothing
-    
+
     -- | like ('<')
     lt    :: a -> a -> Maybe Bool
     lt x y = case x `cmp` y of
              Just LT -> Just True
              Just _  -> Just False
              Nothing -> Nothing
-    
+
     -- | like 'max'. The default instance returns the left argument
     -- when they're equal.
     maxPO    :: a -> a -> Maybe a
@@ -96,7 +100,7 @@ class PartialOrd a where
                        GT -> Just x
                        EQ -> Just x
                        LT -> Just y
-    
+
     -- | like 'min'. The default instance returns the left argument
     -- when they're equal.
     minPO    :: a -> a -> Maybe a
@@ -108,7 +112,11 @@ class PartialOrd a where
 
 infix 4 `gt`, `ge`, `eq`, `ne`, `le`, `lt`, `maxPO`, `minPO`
 
-instance (Ord a) => PartialOrd a where
+instance
+#if __GLASGOW_HASKELL__ >= 710
+    {-# OVERLAPPABLE #-}
+#endif
+    (Ord a) => PartialOrd a where
     cmp   x y = Just $! x `compare` y
     gt    x y = Just $! x >  y
     ge    x y = Just $! x >= y
@@ -126,11 +134,19 @@ instance (Ord a) => PartialOrd a where
 -- returns @Just Eq@ for @notANumber@ then CPP was run wrongly.
 --
 -- The instances inherited from Ord are wrong. So we'll fix them.
-instance PartialOrd Float where
+instance
+#if __GLASGOW_HASKELL__ >= 710
+    {-# OVERLAPPING #-}
+#endif
+    PartialOrd Float where
     cmp x y | isNaN x || isNaN y = Nothing
             | otherwise          = Just $! x `compare` y
 
-instance PartialOrd Double where
+instance
+#if __GLASGOW_HASKELL__ >= 710
+    {-# OVERLAPPING #-}
+#endif
+    PartialOrd Double where
     cmp x y | isNaN x || isNaN y = Nothing
             | otherwise          = Just $! x `compare` y
 
