@@ -1,25 +1,30 @@
 -- Needed to ensure correctness, and because we can't guarantee rules fire
 -- The MagicHash is for unboxed primitives (-fglasgow-exts also works)
 --     We only need MagicHash if on GHC, but we can't hide it in an #ifdef
-{-# LANGUAGE MultiParamTypeClasses
-           , OverlappingInstances
+{-# LANGUAGE CPP
+           , MultiParamTypeClasses
            , FlexibleInstances
-           , CPP
-           , MagicHash
            #-}
 
+#if __GLASGOW_HASKELL__ < 710
+{-# LANGUAGE OverlappingInstances #-}
+#endif
+
+-- We don't put these in LANGUAGE, because it's CPP guarded for GHC only
+{-# OPTIONS_GHC -XMagicHash #-}
+
 {-# OPTIONS_GHC -Wall -fwarn-tabs #-}
 
 ----------------------------------------------------------------
---                                                  ~ 2009.01.29
+--                                                  ~ 2021.10.17
 -- |
 -- Module      :  Data.Number.RealToFrac
--- Copyright   :  Copyright (c) 2007--2010 wren ng thornton
+-- Copyright   :  Copyright (c) 2007--2021 wren gayle romano
 -- License     :  BSD3
--- Maintainer  :  wren@community.haskell.org
+-- Maintainer  :  wren@cpan.org
 -- Stability   :  stable
 -- Portability :  semi-portable (CPP, MPTC, OverlappingInstances)
--- 
+--
 -- This module presents a type class for generic conversion between
 -- numeric types, generalizing @realToFrac@ in order to overcome
 -- problems with pivoting through 'Rational'
@@ -75,7 +80,11 @@ class (Real a, Fractional b) => RealToFrac a b where
 instance (Real a, Fractional a) => RealToFrac a a where
     realToFrac = id
 
-instance (Real a, Transfinite a, Fractional b, Transfinite b)
+instance
+#if __GLASGOW_HASKELL__ >= 710
+    {-# OVERLAPPABLE #-}
+#endif
+    (Real a, Transfinite a, Fractional b, Transfinite b)
     => RealToFrac a b
     where
     realToFrac x
@@ -86,31 +95,55 @@ instance (Real a, Transfinite a, Fractional b, Transfinite b)
 
 
 #ifdef __GLASGOW_HASKELL__
-instance RealToFrac Int Float where
+instance
+#if __GLASGOW_HASKELL__ >= 710
+    {-# OVERLAPPING #-}
+#endif
+    RealToFrac Int Float where
     {-# INLINE realToFrac #-}
     realToFrac (I# i) = F# (int2Float# i)
 
-instance RealToFrac Int Double where
+instance
+#if __GLASGOW_HASKELL__ >= 710
+    {-# OVERLAPPING #-}
+#endif
+    RealToFrac Int Double where
     {-# INLINE realToFrac #-}
     realToFrac (I# i) = D# (int2Double# i)
 
 
-instance RealToFrac Integer Float where
+instance
+#if __GLASGOW_HASKELL__ >= 710
+    {-# OVERLAPPING #-}
+#endif
+    RealToFrac Integer Float where
     -- TODO: is there a more primitive way?
     {-# INLINE realToFrac #-}
     realToFrac j = Prelude.realToFrac j
 
-instance RealToFrac Integer Double where
+instance
+#if __GLASGOW_HASKELL__ >= 710
+    {-# OVERLAPPING #-}
+#endif
+    RealToFrac Integer Double where
     -- TODO: is there a more primitive way?
     {-# INLINE realToFrac #-}
     realToFrac j = Prelude.realToFrac j
 
 
-instance RealToFrac Float Double where
+instance
+#if __GLASGOW_HASKELL__ >= 710
+    {-# OVERLAPPING #-}
+#endif
+    RealToFrac Float Double where
     {-# INLINE realToFrac #-}
     realToFrac (F# f) = D# (float2Double# f)
-    
-instance RealToFrac Double Float where
+
+instance
+#if __GLASGOW_HASKELL__ >= 710
+    {-# OVERLAPPING #-}
+#endif
+    RealToFrac Double Float where
     {-# INLINE realToFrac #-}
     realToFrac (D# d) = F# (double2Float# d)
 #endif
