@@ -1,29 +1,33 @@
 {-# OPTIONS_GHC -Wall -fwarn-tabs #-}
+{-# OPTIONS_GHC -O2 -fenable-rewrite-rules #-}
 
--- Unfortunately we need -fglasgow-exts in order to actually pick
--- up on the rules (see -ddump-rules). The -frewrite-rules flag
--- doesn't do what you want.
--- <http://hackage.haskell.org/trac/ghc/ticket/2213>
--- <http://www.mail-archive.com/glasgow-haskell-users@haskell.org/msg14313.html>
-{-# OPTIONS_GHC -fglasgow-exts #-}
-
+-- FIXME(2023-03-19): Since recent versions of GHC the rewrite rules
+-- in this file generate warnings that they may never fire because
+-- the rule "Class op exp" may fire first.  Although the warning
+-- suggests adding a phase limit, that doesn't actually help because
+-- the "Class op exp" rule is a built-in.  So it's not actually
+-- clear how to silence this warning:
+-- <https://gitlab.haskell.org/ghc/ghc/-/issues/10595>
+--
+-- So for now we just silence the warnings.
+{-# OPTIONS_GHC -fno-warn-inline-rule-shadowing #-}
 ----------------------------------------------------------------
---                                                  ~ 2009.03.09
+--                                                  ~ 2021.10.17
 -- |
 -- Module      :  Data.Number.Transfinite
--- Copyright   :  Copyright (c) 2007--2010 wren ng thornton
+-- Copyright   :  Copyright (c) 2007--2021 wren gayle romano
 -- License     :  BSD3
--- Maintainer  :  wren@community.haskell.org
+-- Maintainer  :  wren@cpan.org
 -- Stability   :  stable
 -- Portability :  portable
--- 
+--
 -- This module presents a type class for numbers which have
 -- representations for transfinite values. The idea originated from
 -- the IEEE-754 floating-point special values, used by
 -- "Data.Number.LogFloat". However not all 'Fractional' types
 -- necessarily support transfinite values. In particular, @Ratio@
 -- types including 'Rational' do not have portable representations.
--- 
+--
 -- For the Glasgow compiler (GHC 6.8.2), "GHC.Real" defines @1%0@
 -- and @0%0@ as representations for 'infinity' and 'notANumber',
 -- but most operations on them will raise exceptions. If 'toRational'
@@ -36,7 +40,7 @@
 -- * <http://www.haskell.org/pipermail/haskell-prime/2006-February/000791.html>
 --
 -- * <http://www.haskell.org/pipermail/haskell-prime/2006-February/000821.html>
--- 
+--
 -- Hugs (September 2006) stays closer to the haskell98 spec and
 -- offers no way of constructing those values, raising arithmetic
 -- overflow errors if attempted.
@@ -75,24 +79,24 @@ import Data.Number.PartialOrd
 -- is compiled correctly.
 
 class (PartialOrd a) => Transfinite a where
-    
+
     -- | A transfinite value which is greater than all finite values.
     -- Adding or subtracting any finite value is a no-op. As is
     -- multiplying by any non-zero positive value (including
     -- @infinity@), and dividing by any positive finite value. Also
     -- obeys the law @negate infinity = negativeInfinity@ with all
     -- appropriate ramifications.
-    
+
     infinity :: a
-    
-    
+
+
     -- | A transfinite value which is less than all finite values.
     -- Obeys all the same laws as @infinity@ with the appropriate
     -- changes for the sign difference.
-    
+
     negativeInfinity :: a
-    
-    
+
+
     -- | An exceptional transfinite value for dealing with undefined
     -- results when manipulating infinite values. The following
     -- operations must return @notANumber@, where @inf@ is any value
@@ -112,11 +116,11 @@ class (PartialOrd a) => Transfinite a where
     --
     -- * @inf \/ inf@
     --
-    -- * @inf `div` inf@
+    -- * @inf \`div\` inf@
     --
     -- * @0 \/ 0@
     --
-    -- * @0 `div` 0@
+    -- * @0 \`div\` 0@
     --
     -- Additionally, any mathematical operations on @notANumber@
     -- must also return @notANumber@, and any equality or ordering
@@ -125,14 +129,14 @@ class (PartialOrd a) => Transfinite a where
     -- for 'Eq'; thus, 'eq' and 'ne' are preferred over ('==') and
     -- ('/=')). Since it returns false for equality, there may be
     -- more than one machine representation of this `value'.
-    
+
     notANumber :: a
-    
-    
+
+
     -- | Return true for both @infinity@ and @negativeInfinity@,
     -- false for all other values.
     isInfinite :: a -> Bool
-    
+
     -- | Return true only for @notANumber@.
     isNaN      :: a -> Bool
 
@@ -163,7 +167,7 @@ instance Transfinite Float where
 -- This function will raise an error when taking the log of negative
 -- numbers, rather than returning 'notANumber' as the newer GHC
 -- implementation does. The reason being that typically this is a
--- logical error, and @notANumber@ allows the error to propegate
+-- logical error, and @notANumber@ allows the error to propagate
 -- silently.
 --
 -- In order to improve portability, the 'Transfinite' class is
@@ -184,8 +188,8 @@ instance Transfinite Float where
 -- be noted in case your code depends on the implementation details.
 
 log  :: (Floating a, Transfinite a) => a -> a
-{-# SPECIALIZE log :: Double -> Double #-}
-{-# SPECIALIZE log :: Float  -> Float  #-}
+{-# INLINE [0] log #-}
+-- TODO: should we use NOINLINE or [~0] to avoid the possibility of code bloat?
 log x = case x `cmp` 0 of
         Just GT -> Prelude.log x
         Just EQ -> negativeInfinity
@@ -201,12 +205,17 @@ log x = case x `cmp` 0 of
 
 ----------------------------------------------------------------
 -- These rules moved here from "LogFloat" in v0.11.2
+--
+-- FIXME(2023-03-19): Since recent versions of GHC these rules
+-- generate warnings that they may never fire because the rule
+-- "Class op exp" may fire first.  Although the warning suggests
+-- adding a phase limit, that doesn't actually help because the
+-- "Class op exp" rule is a built-in.  So it's not actually clear
+-- how to silence this warning:
+-- <https://gitlab.haskell.org/ghc/ghc/-/issues/10595>
 {-# RULES
 "log/exp"  forall x. log (exp x) = x
-"log.exp"            log . exp   = id
-
 "exp/log"  forall x. exp (log x) = x
-"exp.log"            exp . log   = id
     #-}
 
 -- We'd like to be able to take advantage of general rule versions
