@@ -11,15 +11,15 @@
 #define REALFLOAT_VERSION normal Prelude version. This could be buggy.
 #endif
 ----------------------------------------------------------------
---                                                  ~ 2010.03.19
+--                                                  ~ 2021.10.17
 -- |
 -- Module      :  Hugs.RealFloat
--- Copyright   :  Copyright (c) 2007--2010 wren ng thornton
+-- Copyright   :  Copyright (c) 2007--2021 wren gayle romano
 -- License     :  BSD3
--- Maintainer  :  wren@community.haskell.org
+-- Maintainer  :  wren@cpan.org
 -- Stability   :  stable
 -- Portability :  portable (with CPP)
--- 
+--
 -- Hugs (September 2006) has buggy definitions for 'Prelude.isNaN'
 -- and 'Prelude.isInfinite' on Float and Double. If this module is
 -- run through CPP with the macro @__HUGS__@ set to a value no
@@ -44,9 +44,7 @@ import Prelude hiding (isInfinite, isNaN)
 import qualified Prelude
 ----------------------------------------------------------------
 
-isInfinite  :: (RealFloat a) => a -> Bool
-{-# SPECIALIZE isInfinite :: Double -> Bool #-}
-{-# SPECIALIZE isInfinite :: Float  -> Bool #-}
+isInfinite :: (RealFloat a) => a -> Bool
 {-# INLINE isInfinite #-}
 #if defined(__HUGS__) && (__HUGS__ <= 200609)
 isInfinite x = (1/0) == abs x
@@ -56,8 +54,6 @@ isInfinite = Prelude.isInfinite
 
 
 isNaN :: (RealFloat a) => a -> Bool
-{-# SPECIALIZE isNaN :: Double -> Bool #-}
-{-# SPECIALIZE isNaN :: Float  -> Bool #-}
 {-# INLINE isNaN #-}
 #if defined(__HUGS__) && (__HUGS__ <= 200609)
 isNaN x = compareEQ x 0 && compareEQ x 1
@@ -65,10 +61,11 @@ isNaN x = compareEQ x 0 && compareEQ x 1
 -- | In Hugs (September 2006), 'compare' always returns @EQ@ if one
 -- of the arguments is not a number. Thus, if a number is @compareEQ@
 -- against multiple different numbers, then it must be @isNaN@.
-compareEQ    :: (Ord a) => a -> a -> Bool
-compareEQ x y = case compare x y of
-                EQ -> True
-                _  -> False
+compareEQ :: (Ord a) => a -> a -> Bool
+compareEQ x y =
+    case compare x y of
+    EQ -> True
+    _  -> False
 #else
 isNaN = Prelude.isNaN
 #endif
